#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, re
from urlparse import urlparse
from PyQt4 import QtCore
from PyQt4.QtNetwork import QNetworkRequest, QNetworkAccessManager, QNetworkProxy, QNetworkReply
import HTMLParser
from utils.DebugLog import DebugLog, DebugFile
from model.RideModel import RideModel, Ride

class Extractor(QtCore.QObject):
   def __init__(self, id):
      DebugLog("Extractor::Init")
      QtCore.QObject.__init__(self)

      self.url = ''
      self.urlPerturb = ''
      self.rideModel = RideModel(id)
      self.parser = None
      self.treestore = None
      self.max = 0
      self.fetching = False
      self.id = id

      self.online = False
      self.manager = None
      self.currentRide = None
      self.parsePerturbation =  False

   def Clear(self):
      self.listRides = []
      self.perturbation = ""
      self.error = ""
      self.currentRide = None

   def SetOnline(self, b):
      self.online = b

   def InitNetwork(self):
#      DebugLog("Extractor::InitNetwork")

      self.manager = QNetworkAccessManager()
      self.connect(self.manager, QtCore.SIGNAL("finished(QNetworkReply *)"), self.readResponse )

      proxy = os.environ.get('PROXY')
      if proxy:
         o = urlparse(proxy)
         netlocL = o.netloc.find(':')
         host = o.netloc[:netlocL]
#         DebugLog("  - Using proxy : %s://%s:%d"%(o.scheme, host, o.port))
         self.proxy = QNetworkProxy(QNetworkProxy.HttpProxy, host, o.port)
         self.manager.setProxy(self.proxy)

   def SetOnline(self, b):
      self.online = b
      if self.online:
         self.InitNetwork()

   def FetchDatas(self, model, max):
      DebugLog("Extractor::FetchDatas %s"%self.id)
      if self.fetching:
         DebugLog("  Already fetching")
         return

      self.fetching = True
      self.max = max
      self.model = model
      self.rideModel.Clear()

      if self.online and self.manager:
         request = QNetworkRequest( QtCore.QUrl(self.url) )
         reply = self.manager.get(request)
         self.connect(reply, QtCore.SIGNAL("downloadProgress(qint64, qint64)"), self.updateBar)

      else:
         data = open('HSL-EM.htm', 'r').read()
         #data = open('LaTourMaubourg8A.htm').read()

         self.Parse(data)

   def updateBar(self, read,total):
      self.rideModel.SetProgress(read, total)

   def readResponse(self, reply):
      DebugLog("Extractor::readResponse %s"%self.id)
      error = reply.error()

      # test d'erreur, en général, pas de connection internet
      if error != QNetworkReply.NoError:
         self.rideModel.SetInfo('ERROR', u"Erreur, vérifiez votre connection Internet")

      html = str(reply.readAll())
      #DebugFile(html)

      if not self.parsePerturbation:
         self.Parse(html)
      else:
         self.ParsePerturbation(html)
         self.parsePerturbation = False

      self.emit(QtCore.SIGNAL('extracted'), self.rideModel)
      self.fetching = False

   def Parse(self, str):
      DebugLog("Extractor::Parse %s"%self.id)
      
      try:
         self.Feed( str )
      except HTMLParser.HTMLParseError, e:
         print e

      perturbation, error = self.fillModel(self.max)

      if perturbation and self.online:
         self.parsePerturbation =  True
         request = QNetworkRequest( QtCore.QUrl( self.urlPerturb ) )
         self.manager.get(request)

      if error:
         self.rideModel.SetInfo('ERROR', error)

   def ParsePerturbation(self, str):
      pass

   def AppendRidesToModel(self, imax):
      DebugLog("Extractor::AppendRidesToModel %s"%self.id)
      DebugLog("  len(self.listRides)=%d"%len(self.listRides))
      if len(self.listRides)==0:
         DebugFile(self.html, 'html_%s.html'%self.id)
      cpt = 0
      for ride in self.listRides:
         self.rideModel.addRide(ride)
         cpt += 1
         if imax>0 and cpt>=imax:
            break
